/*
 * GProFTPD - A GTK+ frontend for the ProFTPD standalone server.
 * Copyright (C) 2001 - 2006 Magnus Loef (Magnus-swe) <magnus-swe@telia.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 *
 */



#include "../config.h"
#include <gtk/gtk.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "widgets.h"
#include "gettext.h"
#include "allocate.h"
#include "show_info.h"

extern int MAX_READ_POPEN;

extern char global_version[1024];


/* Transforms DisplayFirstChdir to DisplayChdir (proftpd-1.3.0)
   Remove this function ~ 2007-10-12 */
void transform_displayfirstchdir_option(struct w *widgets)
{
    FILE *fp;
    char *line, *new_conf;
    long file_size;
    int transformed = 0;
    gchar *info;

    if((fp=fopen(PROFTPD_CONF, "r"))==NULL)
    {
	printf("Transforming configuration options failed.\n");
        return;
    }
    fseek(fp, 0, SEEK_END);
    file_size = ftell(fp);
    rewind(fp);
    
    line = allocate(file_size+1);
    new_conf = allocate(file_size+16384);
    
    while(fgets(line, file_size, fp)!=NULL)
    {
       if( strstr(line, "DisplayFirstChdir") )
       {
	    transformed = 1;
	    strcat(new_conf, "DisplayChdir .msg\n");
       }
       else
         strcat(new_conf, line);
    }
    fclose(fp);
    
    /* Write the changed conf */
    if((fp=fopen(PROFTPD_CONF, "w+"))==NULL)
    {
	printf("Transforming configuration options failed.\n");
        return;
    }
    fputs(new_conf, fp);    
    fclose(fp);

    if( transformed )
    {
	info = g_strdup_printf(_("Transformed configuration option:\nDisplayFirstChdir to DisplayChdir\n"));
	show_info(info);
	g_free(info);
    }
}


/* Removes the deprecated "bind " option */
void remove_bind_option(struct w*widgets)
{
    FILE *fp;
    char *line, *new_conf;
    long file_size;
    int transformed = 0;
    gchar *info;

    if((fp=fopen(PROFTPD_CONF, "r"))==NULL)
    {
	printf("Transforming configuration options failed.\n");
        return;
    }
    fseek(fp, 0, SEEK_END);
    file_size = ftell(fp);
    rewind(fp);
    
    line = allocate(file_size+1);
    new_conf = allocate(file_size+1);
    
    while(fgets(line, file_size, fp)!=NULL)
    {
       if( (strstr(line, "bind") || strstr(line, "Bind")) && line[4]==' ' )
	 transformed = 1;
       else
         strcat(new_conf, line);
    }
    fclose(fp);
    
    /* Write the changed conf */
    if((fp=fopen(PROFTPD_CONF, "w+"))==NULL)
    {
	printf("Transforming configuration options failed.\n");
        return;
    }
    fputs(new_conf, fp);    
    fclose(fp);

    if( transformed )
    {
	info = g_strdup_printf(_("Removed configuration option: \"Bind \"\nAs it has been deprecated."));
	show_info(info);
	g_free(info);
    }
}


void set_version(struct w *widgets)
{
    FILE *fp;
    char *version;
    int ver_found = 0;
    gchar *command;
    gchar *ver=NULL;
    gchar *utf8=NULL;
    gchar *info;

    command = g_strdup_printf("%s -v 2>&1", PROFTPD_BINARY);
    if((fp=popen(command, "r"))==NULL)
    {
       info = g_strdup_printf(_("Set version failed\n"));
       show_info(info);
       g_free(info);
       g_free(command);
       return;
    }

    g_free(command);    
    version = allocate(MAX_READ_POPEN);
    
    while(fgets(version, MAX_READ_POPEN, fp)!=NULL)
    {
       if( strstr(version, "ProFTPD") && strlen(version) < 30 )
       {
	  /* Cut the first 3 chars " _ " */
          ver = g_strdup_printf(_(" Information: %s"), &version[3]);
	  utf8 = g_locale_to_utf8(ver, strlen(ver)-1, NULL, NULL, NULL);
          gtk_label_set_text(GTK_LABEL(widgets->version_label), utf8);
	  g_free(utf8);
	  ver_found = 1;
       }
    }
    pclose(fp);

    if( ! ver_found )
    {
        ver = g_strdup_printf(_(" Proftpd is not installed or not in your path."));
	utf8 = g_locale_to_utf8(ver, strlen(ver), NULL, NULL, NULL);
        gtk_label_set_text(GTK_LABEL(widgets->version_label), utf8);
	g_free(utf8);
	g_free(ver);
    }

    /* Used to determine what options to use with various versions of proftpd */
    snprintf(global_version, 1000, "%s", version);

    /* Transform DisplayFirstChdir to DisplayChdir in proftpd version >= 1.3.1 */
    if( ver_found && ! strstr(version, "1.1.")
    && ! strstr(version, "1.2.") && ! strstr(version, "1.3.0") )
    {
	transform_displayfirstchdir_option(widgets);

	/* Also remove the "bind " option */	
	remove_bind_option(widgets);
    }

    free(version);
}
